;***********************************************************************
;* Initialition ATMega8535  and basic macro                                *
;*                                                                     *
;* Vladimir Visnovsky                                                  *
;***********************************************************************

; Frekvencie	.DEFINE CPUFreqenci xxx
.equ f1Mhz		= 1
.equ f8Mhz		= 2
.equ f16Mhz		= 4 
 
.Message "Ceated by: Vladimir Visnovsky "
 
; Ak nieje prednastaveny ziadny typ procesora a frekvencie 
; prednasyavia sa tieto hodnoty
; - Cpu Frequencia
.ifndef 	CpuFreqenci
 .define	CpuFreqenci f8Mhz
.endif

; === Registre ===
 ; Zakladne registre
; .def	XH	= r27
; .def	XL	= r26
; .def	YH	= r29
; .def	YL	= r28
; .def	ZH	= r31
; .def	ZL	= r30
 ; Pre procedury
; .def	A	= r16
; .def	B	= r17
; .def	T	= r18
; .def	H	= r25
; .def	L	= r24

; === Zakladne makra ===
; - Acc [Byte]		; Vlozi do A cislo Byte
; - Acc [Register]	; Prekopiruje z Registra do A
; - Parameter [Byte/Register] je presne ako Acc
; - ParameterHL,X,Y,Z [Word/RH,RL/RH:RL]
; - Address [adrersa] ; Do Z vlozi adresu uz prepocitanu o x2
;
; === Makra zasobnika ===
; - PushA	; Ulozenie registrov SREG, A, T do zasobnika
; - PopA	; Vytiahnutie zo zasobnika SREG, A, T
;   Interupt_Timer:
;    PushA
;      :
;    PopA
;    RetI
;
; === Makra Portov ===
; - Port"X"_Init [Byte] ; Nastavy smer portu 1-vystup/0-vstup
;                     ; Nastavy vystup portu na negovanu hodnotu smeru.
; - Port"X"_Out [Byte]  ; Na port zapise Byte
; - Port"X"_Out         ; Na port zapise hodnotu z A (r16)
; - Port"X"_Load        ; Nacita hodnotu z Pinou portu a vlozi do A (r16)
;   + Za "X" sa dosadi oznacenie portu. A, B, C, D. 
;   + !!! PortA je pouzitelny iba pre uP v DIP40
; - PullUp_Enable		; Zapnutie PullUp rezistorov
; - PullUp_Disable		; Vypnutie PullUp rezistorov
;  InitPort:
;   PortD_Init [0b11110000]
; 	PortD_Out  [0xFF]
;     :
;   PortD_Load
;   swap	A
;   PortD_Out
;
; === Makra prerusenia ===
; Globalne prerusenie
; - Interupt_Enable 	; Globalne povolenie prerusenia
; - Interupt_Disable	; Globalne zakazanie prerusenia
; Externe prerusenie x=0,1,2 (2 iba pre DIP40)
; - INTx_Enable			; Povolenie prerusenia
; - INTx_Disable		; Zakazanie prerusenia
; - INTx_StayOnL		; Prerusenie od L na pine (nepodporuje INT2)
; - INTx_OnL			; Prerusenie od H->L na pine
; - INTx_OnH			; Prerusenie od L->H na pine
; - INTx_OnChange		; Prerusenie od zmeny urovne (nepodporuje INT2)
; - INTx_Init			; Prestavy pin na vstup, INTx_OnH, INTx_Enable
; - Interupt_INTx:		; Adresa prerusenia 
;
;  Init:
;   INT0_Init		; INT0 spusteny a reaguje na zmenu z L do H
; 	INT1_OnChange	; INT1 nastavy na reagovanie od zmeny
;	INT1_Enable		; Povolenie INT1
;    :
;   Interupt_Enable ; Povolenie prerusenia (Vseobecne)
; 	 :
;
;  Interupt_INT0:	: Prerusenie od INT0
;    :
;   Reti
;
;  Interupt_INT1:	; Prerusenie od INT1
;    :
;   Reti		
;
; === Timer ====
; - Timer_Init		; Inicializuje 16 bitovy timer
;                     - CTC mod, preddelicka na 8
;                     - Top - nastaveny z registrov HL
; - Timer_Init [Top: Word, Div: Byte (0-7)]
;                   ; Top - dvojbytove slovo
;                   ; Div - predelicka
; 					; - Timer_Disable	= 0
;					; - Timer_Div1		= 1
;					; - Timer_Div8		= 2
;					; - Timer_Div64		= 3
;					; - Timer_Div256	= 4
;					; - Timer_Div1024	= 5
;					; - Timer_T1		= 6
;					; - Timer_NegT1		= 7 
; - Timer_Reset		; Restartovanie Pocitadla citaca
; - Timer_Top		; Nastavenie Topu citaca. Top <- HL
; - Timer_Top [Top: Word] 
;                   ; Nastavenie Topu citaca. Top je dvojbytove slovo
; - Timer_Div [Div: Byte]
;                   ; Nastavenie predelicky. konstanty vyssie
; - Timer_Enable/Disable
;
; === Timer 8 ===
; - Timer8_Init		; Inicializuje 16 bitovy timer
;                     - CTC mod, preddelicka na 8
;                     - Top - nastaveny z registra L
; - Timer8_Init [Top: Byte, Div: Byte (0-7)]
;                   ; Top - jednobytove slovo
;                   ; Div - predelicka
; 					; - Timer_Disable	= 0
;					; - Timer_Div1		= 1
;					; - Timer_Div8		= 2
;					; - Timer_Div32		= 3
;					; - Timer_Div64		= 4
;					; - Timer_Div128	= 5
;					; - Timer_Div256	= 6
;					; - Timer_Div1024	= 7 
; - Timer8_Reset	; Restartovanie Pocitadla citaca
; - Timer8_Top		; Nastavenie Topu citaca. Top <- L
; - Timer8_Top [Top: Byte] 
;                   ; Nastavenie Topu citaca. Top je jednobytove slovo
; - Timer8_Div [Div: Byte]
;                   ; Nastavenie predelicky. konstanty vyssie
;
; === EEPROM ===
; - EEPROM_Wait		; Caka, kym EEPROM dopracuje
; - EEPROM_Write    ; Ulozi do EEPROM na adrese HL data z A
; - EEPROM_Write [Adresa : Word] 
;                   ; Ulozi do EEPROM na adrese data z A
; - EEPROM_Read     ; Nacita z EEPROM na "adrese" HL data do A
; - EEPROM_Read [Adresa : Word] 
;                   ; Nacita z EEPROM na "adrese" data do A
;
; === AD prevodnik ===
; - AD_Ref5V		; referencne napatie nastavi na 5V
; - AD_Ref2V56		; referencne napatie nastavi na 2,56V
; - AD_ARef			; referencia zo vstupu ARef (prednastavene)
; - AD_Convert		; A je cilo pinu (mux) vystup je 8bit v A
; - AD_Convert [MUX]; Mux je cilo pinu (mux) vystup je 8bit v A
; - AD_Convert [MUX: Byte or register; OUT: Register r16-r31]
; - AD_Convert_10	; A je cilo pinu (mux) vystup je 10bit v HL
; - AD_Convert_10 [MUX]
;					; Mux je cilo pinu (mux) vystup je 10bit v HL
; ADInit: 
;  ; Referencne napatie 5V
;  AD_Ref5V	
;  ; Makro pre volanie konverzie
;  ; - AD_Read [MUX : Byte]
;  .Macro AD_Read_i
;   Ldi	A,@0
;   RCall ADConvert
;  .EndMacro
;
;    : 
;
; ADConvert:
;  AD_Convert
;  Ret
;
; === USART ===
; - USART_Init [UBRR : Byte]	; Inicializacia USARTu. UBRR:
;                           ; StopBit1, TxEnable, RxEnable
;                         	; U2400,U4800,U9600,U19K2,U38K4,U250K,U500k
; - USART_Send 				; Posli byte A-> USART TX
; - USART_Send [Data : Byte/register]
; - USART_Read				; Nacitaj byte USART RX -> A
; - USART_Read [Data : Register]
; - USART_Start 				; Spustenie RX aj TX
; - USART_Rx_Start 
; - USART_Tx_Start 
; - USART_Stop				; Stopnutie RX aj TX
; - USART_Rx_Stop
; - USART_Tx_Stop
;  Utility					; Podmienene skoky
; - USART_SkipIfNot_FrameError
; - USART_SkipIf_DataOverRun
; - USART_SkipIfNot_DataOverRun
; - USART_ResetFlag_FrameError
; - USART_ResetFlag_DataOverRun
;  Nastavenia				; Povolenie a zakazanie preruseni
; - USART_Interupt_RX_Enable
; - USART_Interupt_RX_Disable
; - USART_Interupt_TX_Enable
; - USART_Interupt_TX_Disable
; 
; === Delay ===
; - Delay_us [word]			; Pockaj mikrosenund max. 4095
; - Delay_Parameter [word]	; Prepise Word -> HL cez korekciu frekvencie
; - Delay_Body				; Pockaj mikrosekund z HL
;
; .Macro Delay_i
;  Delay_Parameter [@0]
;  RCall Delay_Adress
; .EndMacro
;
; Delay_Adress:
;  Delay_Body
;  ret;
;
; === POKE/PEEK ===
; Vlozenie do RAM
; - Poke (Rx:register) ; ak nieje parameter Rx tak je udaj ulozeny z A
; - Peek (Rx:register) ; ak nieje parameter Rx tak je udaj ulozeny do A
;
; ==== I2C ====
; I2C_Start
; I2C_Stop
; I2C_SendAdr [Address : Byte]  (BP:A) - Alternativne I2C_Adress
; I2C_SendData [Dat : Byte]     (BP:A) - Alternativne I2C_Send
; I2C_ReadData : Byte v A			   - Alternativne I2C_Read
; I2C_Next : Byte v A

; ------------------------------------------------------------------- 1.
; Inicializacia Cpu
 .device 	ATmega8					; Instrukcna sada Mega8
 .message 	"CPU: ATMega8535"

.if 		CpuFreqenci = f1Mhz
 .message	"Freqenci: 1Mhz"	
.elif 		CpuFreqenci = f8Mhz
 .message	"Freqenci: 8Mhz"	
.elif 		CpuFreqenci = f16Mhz
 .message	"Freqenci: 16Mhz"	
.endif

; ------------------------------------------------------------------- 2.
; Konstanty:
 .equ	FLASHEND	= 0x0fff	; Note: Word address
 .equ	IOEND		= 0x003f
 .equ	SRAM_START	= 0x0060
 .equ	SRAM_SIZE	= 512
 .equ	RAMEND		= 0x025f
 .equ	XRAMEND		= 0x0000
 .equ	E2END		= 0x01ff
 .equ	EEPROMEND	= 0x01ff
 .equ	EEADRBITS	= 9
 ; -- Rozdielne premenne
  ; Velkost tabulky
  .equ	INT_VECTORS_SIZE	= 21	; size in words 
 .equ	OCR0	= 0x3c
 .equ	PORTA	= 0x1b
 .equ	DDRA	= 0x1a
 .equ	PINA	= 0x19
  ; ADMUX
  .equ	MUX4	= 4		; Analog Channel and Gain Selection Bits
  ; SFIOR
  .equ	ADTS0	= 5		; ADC High Speed Mode
  .equ	ADTS1	= 6		; ADC Auto Trigger Source
  .equ	ADTS2	= 7		; ADC Auto Trigger Source
  ; TIMSK
  .equ	OCIE0	= 1		; Timer/Counter0 Output Compare Match Interrupt register
  ; TIFR
  .equ	OCF0	= 1		; Output Compare Flag 0
  ; TCCR0 
  .equ	WGM01	= 3		; Waveform Generation Mode 1
  .equ	CTC0	= WGM01	; For compatibility
  .equ	COM00	= 4		; Compare match Output Mode 0
  .equ	COM01	= 5		; Compare Match Output Mode 1
  .equ	WGM00	= 6		; Waveform Generation Mode 0
  .equ	PWM0	= WGM00	; For compatibility
  .equ	FOC0	= 7		; Force Output Compare
  ; GICR
  .equ	INT2	= 5		; External Interrupt Request 2 Enable
  ; GIFR
  .equ	INTF2	= 5		; External Interrupt Flag 2
  ; MCRSCR
  .equ	ISC2	= 6	; Interrupt Sense Control 2
  ; MCUCR
  .equ	SE	= 6	; Sleep Enable
  .equ	SM2	= 7	; Sleep Mode Select
 ; - Spolocne konstanty
 .equ	SREG	= 0x3f
  ; SREG 
  .equ	SREG_C	= 0		; Carry Flag
  .equ	SREG_Z	= 1		; Zero Flag
  .equ	SREG_N	= 2		; Negative Flag
  .equ	SREG_V	= 3		; Two's Complement Overflow Flag
  .equ	SREG_S	= 4		; Sign Bit
  .equ	SREG_H	= 5		; Half Carry Flag
  .equ	SREG_T	= 6		; Bit Copy Storage
  .equ	SREG_I	= 7		; Global Interrupt Enable
  ; SREG_End
 .equ	SPH		= 0x3e
 .equ	SPL		= 0x3d
 .equ	GICR	= 0x3b
  ; GICR
  .equ	GIMSK	= GICR	; For compatibility
  .equ	IVCE	= 0		; Interrupt Vector Change Enable
  .equ	IVSEL	= 1		; Interrupt Vector Select
  .equ	INT0	= 6		; External Interrupt Request 0 Enable
  .equ	INT1	= 7		; External Interrupt Request 1 Enable  
  ; GICR_End
 .equ	GIFR	= 0x3a
  ; GIFR
  .equ	INTF0	= 6	; External Interrupt Flag 0
  .equ	INTF1	= 7	; External Interrupt Flag 1  
  ; GIFR_End
 .equ	TIMSK	= 0x39
  ; TIMSK
  .equ	TOIE0	= 0 	; Timer/Counter0 Overflow Interrupt Enable
  .equ	TOIE1	= 2 	; Timer/Counter1 Overflow Interrupt Enable
  .equ	OCIE1B	= 3 	; Timer/Counter1 Output CompareB Match Interrupt Enable
  .equ	OCIE1A	= 4 	; Timer/Counter1 Output CompareA Match Interrupt Enable
  .equ	TICIE1	= 5 	; Timer/Counter1 Input Capture Interrupt Enable
  .equ	TOIE2	= 6		; Timer/Counter2 Overflow Interrupt Enable
  .equ	OCIE2	= 7		; Timer/Counter2 Output Compare Match Interrupt Enable  
  ; TIMSK_End
 .equ	TIFR	= 0x38
  ; TIFR
  .equ	TOV0	= 0			; Timer/Counter0 Overflow Flag
  .equ	TOV1	= 2			; Timer/Counter1 Overflow Flag
  .equ	OCF1B	= 3			; Output Compare Flag 1B
  .equ	OCF1A	= 4			; Output Compare Flag 1A
  .equ	ICF1	= 5			; Input Capture Flag 1
  .equ	TOV2	= 6			; Timer/Counter2 Overflow Flag
  .equ	OCF2	= 7			; Output Compare Flag 2
  ; TIFR_End
 .equ	SPMCR	= 0x37
  ; SPMCR
  .equ	SPMEN	= 0 	; Store Program Memory Enable
  .equ	PGERS	= 1 	; Page Erase
  .equ	PGWRT	= 2 	; Page Write
  .equ	BLBSET	= 3 	; Boot Lock Bit Set
  .equ	RWWSRE	= 4 	; Read-While-Write Section Read Enable
  .equ	RWWSB	= 6 	; Read-While-Write Section Busy
  .equ	SPMIE	= 7 	; SPM Interrupt Enable
  ; SPMCR_End
 .equ	TWCR	= 0x36
  ; TWCR
  .equ	I2CR	= TWCR		; For compatibility
  .equ	TWIE	= 0			; TWI Interrupt Enable
  .equ	I2IE	= TWIE		; For compatibility
  .equ	TWEN	= 2			; TWI Enable Bit
  .equ	I2EN	= TWEN		; For compatibility
  .equ	ENI2C	= TWEN		; For compatibility
  .equ	TWWC	= 3			; TWI Write Collition Flag
  .equ	I2WC	= TWWC		; For compatibility
  .equ	TWSTO	= 4			; TWI Stop Condition Bit
  .equ	I2STO	= TWSTO		; For compatibility
  .equ	TWSTA	= 5			; TWI Start Condition Bit
  .equ	I2STA	= TWSTA		; For compatibility
  .equ	TWEA	= 6			; TWI Enable Acknowledge Bit
  .equ	I2EA	= TWEA		; For compatibility
  .equ	TWINT	= 7			; TWI Interrupt Flag
  .equ	I2INT	= TWINT		; For compatibility
  ; TWCR_End
 .equ	MCUCR	= 0x35
  ; MCUCR
  .equ	ISC00	= 0		; Interrupt Sense Control 0 Bit 0
  .equ	ISC01	= 1		; Interrupt Sense Control 0 Bit 1
  .equ	ISC10	= 2		; Interrupt Sense Control 1 Bit 0
  .equ	ISC11	= 3		; Interrupt Sense Control 1 Bit 1
  .equ	SM0	= 4	; Sleep Mode Select
  .equ	SM1	= 5	; Sleep Mode Select
  ; MCUCR_End
 .equ	MCUCSR	= 0x34
  ; MCUCSR
  .equ	MCUSR	= MCUCSR	; For compatibility
  .equ	PORF	= 0		; Power-on reset flag
  .equ	EXTRF	= 1		; External Reset Flag
  .equ	BORF	= 2		; Brown-out Reset Flag
  .equ	WDRF	= 3		; Watchdog Reset Flag
  ; MCUCSR_End 
 .equ	TCCR0	= 0x33
  ; TCCR0
  .equ	CS00	= 0			; Clock Select 1
  .equ	CS01	= 1			; Clock Select 1
  .equ	CS02	= 2			; Clock Select 2
  ; TCCR0_End
 .equ	TCNT0	= 0x32
 .equ	OSCCAL	= 0x31
 .equ	SFIOR	= 0x30
  ; SFIOR
  .equ	PSR10	= 0		; Prescaler Reset Timer/Counter1 and Timer/Counter0
  .equ	PSR2	= 1		; Prescaler Reset Timer/Counter2
  .equ	PUD		= 2		; Pull-up Disable
  .equ	ADHSM	= 4		; ADC High Speed Mode
  .equ	ACME	= 3		; Analog Comparator Multiplexer Enable 
  ; SFIOR_End
 .equ	TCCR1A	= 0x2f
  ; TCCR1A
  .equ	WGM10	= 0		; Waveform Generation Mode
  .equ	PWM10	= WGM10	; For compatibility
  .equ	WGM11	= 1		; Waveform Generation Mode
  .equ	PWM11	= WGM11	; For compatibility
  .equ	FOC1B	= 2		; Force Output Compare 1B
  .equ	FOC1A	= 3		; Force Output Compare 1A
  .equ	COM1B0	= 4		; Compare Output Mode 1B, bit 0
  .equ	COM1B1	= 5		; Compare Output Mode 1B, bit 1
  .equ	COM1A0	= 6		; Comparet Ouput Mode 1A, bit 0
  .equ	COM1A1	= 7		; Compare Output Mode 1A, bit 1
  ; TCCR1A_End
 .equ	TCCR1B	= 0x2e
  ; TCCR1B
  .equ	CS10	= 0 	; Prescaler source of Timer/Counter 1
  .equ	CS11	= 1 	; Prescaler source of Timer/Counter 1
  .equ	CS12	= 2 	; Prescaler source of Timer/Counter 1
  .equ	WGM12	= 3 	; Waveform Generation Mode
  .equ	CTC10	= WGM12	; For compatibility
  .equ	CTC1	= WGM12	; For compatibility
  .equ	WGM13	= 4 	; Waveform Generation Mode
  .equ	CTC11	= WGM13	; For compatibility
  .equ	ICES1	= 6 	; Input Capture 1 Edge Select
  .equ	ICNC1	= 7 	; Input Capture 1 Noise Canceler
  ; TCCR1B_End
 .equ	TCNT1H	= 0x2d
 .equ	TCNT1L	= 0x2c
 .equ	OCR1AH	= 0x2b
 .equ	OCR1AL	= 0x2a
 .equ	OCR1BH	= 0x29
 .equ	OCR1BL	= 0x28
 .equ	ICR1H	= 0x27
 .equ	ICR1L	= 0x26
 .equ	TCCR2	= 0x25
  ; TCCR2A
  .equ	CS20	= 0 	; Clock Select bit 0
  .equ	CS21	= 1 	; Clock Select bit 1
  .equ	CS22	= 2 	; Clock Select bit 2
  .equ	WGM21	= 3 	; Waveform Generation Mode
  .equ	CTC2	= WGM21	; For compatibility
  .equ	COM20	= 4 	; Compare Output Mode bit 0
  .equ	COM21	= 5 	; Compare Output Mode bit 1
  .equ	WGM20	= 6 	; Waveform Genration Mode
  .equ	PWM2	= WGM20	; For compatibility
  .equ	FOC2	= 7 	; Force Output Compare
  ; TCCR2A_End
 .equ	TCNT2	= 0x24
 .equ	OCR2	= 0x23
 .equ	ASSR	= 0x22
  ; ASSR
  .equ	TCR2UB	= 0 	; Timer/counter Control Register2 Update Busy
  .equ	OCR2UB	= 1 	; Output Compare Register2 Update Busy
  .equ	TCN2UB	= 2 	; Timer/Counter2 Update Busy
  .equ	AS2		= 3 	; Asynchronous Timer/counter2 
  ; ASSR_End
 .equ	WDTCR	= 0x21
  ; WDTCR
  .equ	WDTCSR	= WDTCR	; For compatibility
  .equ	WDP0	= 0		; Watch Dog Timer Prescaler bit 0
  .equ	WDP1	= 1		; Watch Dog Timer Prescaler bit 1
  .equ	WDP2	= 2		; Watch Dog Timer Prescaler bit 2
  .equ	WDE		= 3		; Watch Dog Enable
  .equ	WDCE	= 4		; Watchdog Change Enable
  .equ	WDTOE	= WDCE	; For compatibility
  ; WDTCR_End  
 .equ	UBRRH	= 0x20
 .equ	UCSRC	= 0x20
  ; UCSRC
  .equ	UCPOL	= 0		; Clock Polarity
  .equ	UCSZ0	= 1		; Character Size Bit 0
  .equ	UCSZ1	= 2		; Character Size Bit 1
  .equ	USBS	= 3		; Stop Bit Select
  .equ	UPM0	= 4		; Parity Mode Bit 0
  .equ	UPM1	= 5		; Parity Mode Bit 1
  .equ	UMSEL	= 6		; USART Mode Select
  .equ	URSEL	= 7		; Register Select
  ; UCSRC_End
 .equ	EEARH	= 0x1f
 .equ	EEARL	= 0x1e
 .equ	EEDR	= 0x1d
 .equ	EECR	= 0x1c
  ; EECR
  .equ	EERE	= 0		; EEPROM Read Enable
  .equ	EEWE	= 1		; EEPROM Write Enable
  .equ	EEMWE	= 2	 	; EEPROM Master Write Enable
  .equ	EEWEE	= EEMWE	; For compatibility
  .equ	EERIE	= 3		; EEPROM Ready Interrupt Enable
  ; EECR_End 
 .equ	PORTB	= 0x18
 .equ	DDRB	= 0x17
 .equ	PINB	= 0x16
 .equ	PORTC	= 0x15
 .equ	DDRC	= 0x14
 .equ	PINC	= 0x13
 .equ	PORTD	= 0x12
 .equ	DDRD	= 0x11
 .equ	PIND	= 0x10
 .equ	SPDR	= 0x0f
 .equ	SPSR	= 0x0e
  ; SPSR
  .equ	SPI2X	= 0		; Double SPI Speed Bit
  .equ	WCOL	= 6		; Write Collision Flag
  .equ	SPIF	= 7		; SPI Interrupt Flag
  ; SPSR_End
 .equ	SPCR	= 0x0d
  ; SPCR
  .equ	SPR0	= 0		; SPI Clock Rate Select 0 
  .equ	SPR1	= 1		; SPI Clock Rate Select 1
  .equ	CPHA	= 2		; Clock Phase
  .equ	CPOL	= 3		; Clock polarity
  .equ	MSTR	= 4		; Master/Slave Select
  .equ	DORD	= 5		; Data Order
  .equ	SPE		= 6		; SPI Enable
  .equ	SPIE	= 7		; SPI Interrupt Enable
  ; SPCR_End
 .equ	UDR		= 0x0c
 .equ	UCSRA	= 0x0b
  ; UCSRA
  .equ	MPCM	= 0		; Multi-processor Communication Mode
  .equ	U2X		= 1		; Double the USART transmission speed
  .equ	UPE		= 2		; Parity Error
  .equ	PE		= UPE	; For compatibility
  .equ	DOR		= 3		; Data overRun
  .equ	FE		= 4		; Framing Error
  .equ	UDRE	= 5		; USART Data Register Empty
  .equ	TXC		= 6		; USART Transmitt Complete
  .equ	RXC		= 7		; USART Receive Complete
  ; UCSRA_End
 .equ	UCSRB	= 0x0a
  ; UCSRB
  .equ	UCR		= UCSRB	; For compatibility
  .equ	TXB8	= 0		; Transmit Data Bit 8
  .equ	RXB8	= 1		; Receive Data Bit 8
  .equ	UCSZ2	= 2		; Character Size
  .equ	CHR9	= UCSZ2	; For compatibility
  .equ	TXEN	= 3		; Transmitter Enable
  .equ	RXEN	= 4		; Receiver Enable
  .equ	UDRIE	= 5		; USART Data register Empty Interrupt Enable
  .equ	TXCIE	= 6		; TX Complete Interrupt Enable
  .equ	RXCIE	= 7		; RX Complete Interrupt Enable  
  ; UCSRB_End
 .equ	UBRRL	= 0x09
 .equ	ACSR	= 0x08
  ; ACSR
  .equ	ACIS0	= 0		; Analog Comparator Interrupt Mode Select bit 0
  .equ	ACIS1	= 1		; Analog Comparator Interrupt Mode Select bit 1
  .equ	ACIC	= 2		; Analog Comparator Input Capture Enable
  .equ	ACIE	= 3		; Analog Comparator Interrupt Enable
  .equ	ACI		= 4		; Analog Comparator Interrupt Flag
  .equ	ACO		= 5		; Analog Compare Output
  .equ	ACBG	= 6		; Analog Comparator Bandgap Select
  .equ	AINBG	= ACBG	; For compatibility
  .equ	ACD		= 7		; Analog Comparator Disable
  ; ACSR_End
 .equ	ADMUX	= 0x07
  ; ADMUX
  .equ	MUX0	= 0		; Analog Channel and Gain Selection Bits
  .equ	MUX1	= 1		; Analog Channel and Gain Selection Bits
  .equ	MUX2	= 2		; Analog Channel and Gain Selection Bits
  .equ	MUX3	= 3		; Analog Channel and Gain Selection Bits
  .equ	ADLAR	= 5		; Left Adjust Result
  .equ	REFS0	= 6		; Reference Selection Bit 0
  .equ	REFS1	= 7		; Reference Selection Bit 1
  ; ADMUX_End
 .equ	ADCSRA	= 0x06
  ; ADSRA
  .equ	ADCSR	= ADCSRA	; For compatibility
  .equ	ADPS0	= 0 	; ADC  Prescaler Select Bits
  .equ	ADPS1	= 1 	; ADC  Prescaler Select Bits
  .equ	ADPS2	= 2 	; ADC  Prescaler Select Bits
  .equ	ADIE	= 3 	; ADC Interrupt Enable
  .equ	ADIF	= 4 	; ADC Interrupt Flag
  .equ	ADFR	= 5 	; ADC  Free Running Select
  .equ	ADSC	= 6 	; ADC Start Conversion
  .equ	ADEN	= 7 	; ADC Enable
  ; ADSRA_End
 .equ	ADCH	= 0x05
 .equ	ADCL	= 0x04
 .equ	TWDR	= 0x03
  ; TWDR
  .equ	I2DR	= TWDR		; For compatibility
  ; TWDR_End
 .equ	TWAR	= 0x02
  ; TWAR
  .equ	I2AR	= TWAR		; For compatibility
  .equ	TWGCE	= 0			; TWI General Call Recognition Enable Bit
  .equ	TWA0	= 1			; TWI (Slave) Address register Bit 0
  .equ	TWA1	= 2			; TWI (Slave) Address register Bit 1
  .equ	TWA2	= 3			; TWI (Slave) Address register Bit 2
  .equ	TWA3	= 4			; TWI (Slave) Address register Bit 3
  .equ	TWA4	= 5			; TWI (Slave) Address register Bit 4
  .equ	TWA5	= 6			; TWI (Slave) Address register Bit 5
  .equ	TWA6	= 7			; TWI (Slave) Address register Bit 6  
  ; TWAR_End
 .equ	TWSR	= 0x01
  ; TWSR
  .equ	I2SR	= TWSR		; For compatibility
  .equ	TWPS0	= 0			; TWI Prescaler
  .equ	TWS0	= TWPS0		; For compatibility
  .equ	I2GCE	= TWPS0		; For compatibility
  .equ	TWPS1	= 1			; TWI Prescaler
  .equ	TWS1	= TWPS1		; For compatibility
  .equ	TWS3	= 3			; TWI Status
  .equ	I2S3	= TWS3		; For compatibility
  .equ	TWS4	= 4			; TWI Status
  .equ	I2S4	= TWS4		; For compatibility
  .equ	TWS5	= 5			; TWI Status
  .equ	I2S5	= TWS5		; For compatibility
  .equ	TWS6	= 6			; TWI Status
  .equ	I2S6	= TWS6		; For compatibility
  .equ	TWS7	= 7			; TWI Status
  .equ	I2S7	= TWS7		; For compatibility  
  ; TWSR_End
 .equ	TWBR	= 0x00
; - Registre
 ; Zakladne registre
 .def	XH	= r27
 .def	XL	= r26
 .def	YH	= r29
 .def	YL	= r28
 .def	ZH	= r31
 .def	ZL	= r30
 ; Pre procedury
 .def	A	= r16
 .def	B	= r17
 .def	T	= r18
 .def   F   = r19
 .def	H	= r25
 .def	L	= r24
 ; Ine pomenovanie registrov
 
; ------------------------------------------------------------------- 3.
; Tabulka prerusenia
 ; -- Definicia procesora ATMega8535
  .org 0000
  Rjmp Begin
; .equ	INT0addr	= 0x0001	; External Interrupt Request 0
;  .org   INT0addr
;  .ifdef Interupt_INT0
;   .message "Event of INT0"
;   RJmp Interupt_INT0
;  .else
;   RetI
;  .endif 
; .equ	INT1addr	= 0x0002	; External Interrupt Request 1
;  .org   INT1addr
;  .ifdef Interupt_INT1
;   .message "Event of INT1"
;   RJmp Interupt_INT1
;  .else
;   RetI
;  .endif 
; .equ	OC2addr		= 0x0003	; Timer/Counter2 Compare Match
;  .org   OC2addr
;  .ifdef Interupt_Timer8
;   .message "Event of OC2 -> Timer8"
;   RJmp Interupt_Timer
;  .else
;  .ifdef Interupt_OC2
;   .message "Event of OC2"
;   RJmp Interupt_OC2
;  .else
;   RetI
;  .endif
;  .endif 
; .equ	OVF2addr	= 0x0004	; Timer/Counter2 Overflow
 ; .org   OVF2addr
;  .ifdef Interupt_OVF2
;   .message "Event of OVF2"
;   RJmp Interupt_OVF2
;  .else
;   RetI
;  .endif 
; .equ	ICP1addr	= 0x0005	; Timer/Counter1 Capture Event
;  .org   ICP1addr
;  .ifdef Interupt_ICP1
;   .message "Event of ICP1"
;   RJmp Interupt_ICP1
;  .else
;   RetI
;  .endif 
; .equ	OC1Aaddr	= 0x0006	; Timer/Counter1 Compare Match A
;  .org   OC1Aaddr
;  .ifdef Interupt_Timer
;   .message "Event of OC1A -> Timer"
;   RJmp Interupt_Timer
;  .else
;  .ifdef Interupt_OC1A
;   .message "Event of OC1A"
;   RJmp Interupt_OC1A
;  .else 
;;   RetI
 ; .endif
 ; .endif 
; .equ	OC1Baddr	= 0x0007	; Timer/Counter1 Compare Match B
;  .org   OC1Baddr
;  .ifdef Interupt_OC1B
;   .message "Event of OC1B"
;   RJmp Interupt_OC1B
;  .else
;   RetI
;;  .endif 
; .equ	OVF1addr	= 0x0008	; Timer/Counter1 Overflow
;  .org   OVF1addr
;  .ifdef Interupt_OVF1
;   .message "Event of OVF1"
;   RJmp Interupt_OVF1
;  .else
;   RetI
;  .endif 
;; .equ	OVF0addr	= 0x0009	; Timer/Counter0 Overflow
;;  .org   OVF0addr
;  .ifdef Interupt_OVF0
;   .message "Event of OVF0"
;   RJmp Interupt_OVF0
;  .else
;   RetI
;  .endif 
; .equ	SPIaddr		= 0x000a	; Serial Transfer Complete
;  .org   SPIaddr
;  .ifdef Interupt_SPI
;   .message "Event of SPI"
;   RJmp Interupt_SPI
;  .else
;   RetI
;  .endif 
; .equ	URXCaddr	= 0x000b	; USART, Rx Complete
;  .org   URXCaddr
;  .ifdef Interupt_USART
;   .message "Event of URXC -> USART"
;   RJmp Interupt_USART
;  .else
 ; .ifdef Interupt_URXC
 ;  .message "Event of URXC"
;   RJmp Interupt_URXC
;  .else
 ;  RetI
;  .endif 
;  .endif
; .equ	UDREaddr	= 0x000c	; USART Data Register Empty
;  .org   UDREaddr
;  .ifdef Interupt_UDRE
;   .message "Event of UDRE"
;   RJmp Interupt_UDRE
;  .else
;   RetI
;  .endif 
; .equ	UTXCaddr	= 0x000d	; USART, Tx Complete
;  .org   UTXCaddr
;  .ifdef Interupt_UTXC
;   .message "Event of UTXC"
;   RJmp Interupt_UTXC
;  .else
;   RetI
;  .endif 
; .equ	ADCCaddr	= 0x000e	; ADC Conversion Complete
;  .org   ADCCaddr
;  .ifdef Interupt_AD
;;   .message "Event of ADCC -> AD"
;   RJmp Interupt_AD
;  .else
;  .ifdef Interupt_UDCC
;   .message "Event of ADCC"
;   RJmp Interupt_ADCC
;  .else
;   RetI
;  .endif 
;  .endif
; .equ	ERDYaddr	= 0x000f	; EEPROM Ready
;  .org   ERDYaddr
;  .ifdef Interupt_ERDY
;   .message "Event of ERDY"
;   RJmp Interupt_ERDY
;  .else
;   RetI
;  .endif 
; .equ	ACIaddr		= 0x0010	; Analog Comparator
;   .org   ACIaddr
;  .ifdef Interupt_ACI
;   .message "Event of ACI"
;   RJmp Interupt_ACI
;  .else
;   RetI
;  .endif 
; .equ	TWIaddr		= 0x0011	; 2-wire Serial Interface
;  .org   TWIaddr
;  .ifdef Interupt_TWI
;   .message "Event of TWI"
;   RJmp Interupt_TWI
;  .else
;   RetI
;  .endif 
; .equ	INT2addr	= 0x0012	; External Interrupt Request 2
;  .Org  INT2addr 
;  .ifdef Interupt_INT2
;   .message "Event of INT2"
;   RJmp Interupt_INT2
;  .else
;   RetI
;  .endif 
; .equ	OC0addr		= 0x0013	; TimerCounter0 Compare Match  
;  .ifdef Interupt_OC0
;   .message "Event of OC0"
;   RJmp Interupt_OC0
;  .else
;   RetI
;  .endif 
; .equ	SPMRaddr	= 0x0014	; Store Program Memory Ready
;  .org   SPMRaddr
;  .ifdef Interupt_SPMR
;   .message "Event of SPMR"
;   RJmp Interupt_SPMR
;  .else
;   RetI
;  .endif ;

; ------------------------------------------------------------------- 4.
; Makra
; - Zakladne
;   Vlozenie do akumulatora A (r16)
.Macro Acc
 .message "Acc: No parameter. Function ignored. Use: Acc [0xXX] or [rX]"
.EndMacro
.Macro Acc_i
 Ldi	A		, @0
.EndMacro
.Macro Acc_8
 Mov	A		, @0
.EndMacro

;   Parameter 8 bit. : Vlozenie do akumulatora A (r16)
.Macro Parameter
 .message "Parameter is A (r16)"
.EndMacro
.Macro Parameter_i
 Ldi	A		, @0
.EndMacro
.Macro Parameter_8
 Mov	A		, @0
.EndMacro
;   Parameter 16 bit. : Vlozenie do akumulatora H (r25), L (r24)
.Macro ParameterHL
 .message "Parameter is H:L (r25:r24) "
.EndMacro
.Macro ParameterHL_i
 Ldi	H		, High(@0)
 Ldi	L		, Low(@0)
.EndMacro
.Macro ParameterHL_8_8
 Mov	H		, @0
 Mov	L		, @1
.EndMacro
.Macro ParameterHL_16
 Mov	H		, @0
 Mov	L		, @1
.EndMacro
;   Parameter 16 bit. : Vlozenie do ZH:ZL
.Macro ParameterZ
 .message "Z <- H:L (r25:r24)"
 Mov	ZH		, H
 Mov	ZL		, L
.EndMacro
.Macro ParameterZ_i
 Ldi	ZH		, High(@0)
 Ldi	ZL		, Low(@0)
.EndMacro
.Macro ParameterZ_8_8
 Mov	ZH		, @0
 Mov	ZL		, @1
.EndMacro
.Macro ParameterZ_16
 Mov	ZH		, @0
 Mov	ZL		, @1
.EndMacro
;   Parameter 16 bit. : Vlozenie do XH:XL
.Macro ParameterX
 .message "X <- H:L (r25:r24)"
 Mov	XH		, H
 Mov	XL		, L
.EndMacro
.Macro ParameterX_i
 Ldi	XH		, High(@0)
 Ldi	XL		, Low(@0)
.EndMacro
.Macro ParameterX_8_8
 Mov	XH		, @0
 Mov	XL		, @1
.EndMacro
.Macro ParameterX_16
 Mov	XH		, @0
 Mov	XL		, @1
.EndMacro
;   Parameter 16 bit. : Vlozenie do YH:YL
.Macro ParameterY
 .message "Y <- H:L (r25:r24)"
 Mov	YH		, H
 Mov	YL		, L
.EndMacro
.Macro ParameterY_i
 Ldi	YH		, High(@0)
 Ldi	YL		, Low(@0)
.EndMacro
.Macro ParameterY_8_8
 Mov	YH		, @0
 Mov	YL		, @1
.EndMacro
.Macro ParameterY_16
 Mov	YH		, @0
 Mov	YL		, @1
.EndMacro
.Macro Address
 .message "Error: Must parameter!"
.EndMacro
.Macro Address_i
 Ldi	ZH		, High(@0<<1)
 Ldi	ZL		, Low(@0<<1)
.EndMacro
 

; --------------------------------------------------- Zasobnikove makra
; - Zasobnikove Makra
.Macro PushA
 Push	A
 In		A		, SREG
 Cli
 Push	A
 Push 	B
 Push	T
 Push   XL
 Push 	XH
 Push	ZL
 Push	ZH
 Push   R0
 Push   R1
 Push   H
 Push   L
.EndMacro 
.Macro PopA
 Pop    L
 Pop    H
 Pop	R1
 Pop	R0
 Pop	ZH
 Pop	ZL
 Pop	XH
 Pop	XL
 Pop	T
 Pop	B
 Pop	A
 Out 	SREG	, A 
 Pop	A
.EndMacro 

; --------------------------------------------------------------- Porty
; - Makra nastavenia portov
;   PORTA (Iba pre puzdro DIP40)
;    - PortA_Init [0b00001111] ; Smer I/O.  
 .Macro PortA_Init
  .message "PortA_Init: No parameter! Please use PortA_Init [0bxxxxxxxx]"
 .EndMacro
 .Macro PortA_Init_i
  .message "PortA is inicialized"
  ; Nastavenie smeru portu
  Ldi	T		, @0
  Out	DDRA	, T
  ; Nastavenie na vytstup negovanu hodnotu smeru
  Com	T
  Out	PortA	, T
 .EndMacro
;   PORTB
;    - PortB_Init [0b00001111] ; Smer I/O.  
 .Macro PortB_Init
  .message "PortB_Init: No parameter! Please use PortB_Init [0bxxxxxxxx]"
 .EndMacro
 .Macro PortB_Init_i
  .message "PortB is inicialized"
  ; Nastavenie smeru portu
  Ldi	T		, @0
  Out	DDRB	, T
  ; Nastavenie na vytstup negovanu hodnotu smeru
  Com	T
  Out	PortB	, T
 .EndMacro
;   PORTC
;    - PortC_Init [0b00001111] ; Smer I/O.  
 .Macro PortC_Init
  .message "PortC_Init: No parameter! Please use PortC_Init [0bxxxxxxxx]"
 .EndMacro
 .Macro PortC_Init_i
  .message "PortC is inicialized"
  ; Nastavenie smeru portu
  Ldi	T		, @0
  Out	DDRC	, T
  ; Nastavenie na vytstup negovanu hodnotu smeru
  Com	T
  Out	PortC	, T
 .EndMacro
;   PORTD
;    - PortD_Init [0b00001111] ; Smer I/O.  
 .Macro PortD_Init
  .message "PortD_Init: No parameter! Please use PortD_Init [0bxxxxxxxx]"
 .EndMacro
 .Macro PortD_Init_i
  .message "PortD is inicialized"
  ; Nastavenie smeru portu
  Ldi	T		, @0
  Out	DDRD	, T
  ; Nastavenie na vytstup negovanu hodnotu smeru
  Com	T
  Out	PortD	, T
 .EndMacro
; Makra vystupov na port
;   PORTA (iba pri 40 pinovom kontroleru)
;    - PortA_Out [0b00001111] ; Vystup na port.  
 .Macro PortA_Out
  .message "PortA_Out: Parameter is A (r16)"
  ; Vystup na port A
  Out	PortA	, A
 .EndMacro
 .Macro PortA_Out_i
  ; Vystup na port
  Ldi	T		, @0
  Out	PortA	, T
 .EndMacro
 .Macro PortA_Out_8
  ; Vystup na port
  Out	PortA	, @0
 .EndMacro
;   PORTB
;    - PortB_Out [0b00001111] ; Vystup na port.  
 .Macro PortB_Out
  .message "PortB_Out: Parameter is A (r16)"
  ; Vystup na port 
  Out	PortB	, A
 .EndMacro
 .Macro PortB_Out_i
  ; Vystup na port
  Ldi	T		, @0
  Out	PortB	, T
 .EndMacro
 .Macro PortB_Out_8
  ; Vystup na port
  Out	PortB	, @0
 .EndMacro
;   PORTC
;    - PortC_Out [0b00001111] ; Vystup na port.  
 .Macro PortC_Out
  .message "PortC_Out: Parameter is A (r16)"
  ; Vystup na port 
  Out	PortC	, A
 .EndMacro
 .Macro PortC_Out_i
  ; Vystup na port
  Ldi	T		, @0
  Out	PortC	, T
 .EndMacro
 .Macro PortC_Out_8
  ; Vystup na port
  Out	PortC	, @0
 .EndMacro 
;   PORTD
;    - PortD_Out [0b00001111] ; Vystup na port.  
 .Macro PortD_Out
  .message "PortD_Out: Parameter is A (r16)"
  ; Vystup na port 
  Out	PortD	, A
 .EndMacro
 .Macro PortD_Out_i
  ; Vystup na port
  Ldi	T		, @0
  Out	PortD	, T
 .EndMacro
 .Macro PortD_Out_8
  ; Vystup na port
  Out	PortD	, @0
 .EndMacro
 ; Makra vstupu do akumulatora A (r16)
;   PORTA (iba pri 40 pinovom uP)
;    - PortA_Load  
 .Macro PortA_Load
  ; Vstup z portu
  In	A		, PinA
 .EndMacro
;   PORTB
;    - PortB_Load  
 .Macro PortB_Load
  ; Vstup z portu
  In	A		, PinB
 .EndMacro
;   PORTC
;    - PortC_Load  
 .Macro PortC_Load
  ; Vstup z portu
  In	A		, PinC
 .EndMacro
;   PORTD
;    - PortD_Load  
 .Macro PortD_Load
  ; Vstup z portu
  In	A		, PinD
 .EndMacro
; PullUP
.Macro PullUp_Enable
 In 	T		, SFIOR
 Andi 	T		, 0b11111011
 Out 	SFIOR	, T
.EndMacro
.Macro PullUp_Disable
 In 	T		, SFIOR
 Ori 	T		, 0b00000100
 Out 	SFIOR	, T
.EndMacro


; ------------------------------------------------- Globalne prerusenie
; Povolenie a zakazanie prerusenia
; - Interupt_Enable - Globalne povolenie prerusenia
.Macro Interupt_Enable
 Sei
.EndMacro
; - Interupt_Disable - Globalne prerusenie zakazane 
.Macro Interupt_Disable
 Cli
.EndMacro

; -------------------------------------------------- Externe prerusenie
; Makra prerusenia
; - INT0
; INT0_Enable - Povoli prerusenie
.Macro INT0_Enable
 Set
 In		T		, GICR
 Bld	T		, INT0
 Out	GICR	, T
.EndMacro
; INT0_Disable - Zakaze prerusenie
.Macro INT0_Disable
 Clt
 In		T		, GICR
 Bld	T		, INT0
 Out	GICR	, T
.EndMacro
; INT0_StayOnL - prerusenie ak je INT v nizkej urovni
.Macro INT0_StayOnL
 In		T		, MCUCR
 ; 1-ISC01, 0-ISC00  // 00
 Andi 	T		, 0b11111100 
 Out	MCUCR	, T	
.EndMacro
; INT0_OnL - prerusenie ak INT vojde do nizkej urovni
.Macro INT0_OnL
 In		T		, MCUCR
 ; 1-ISC01, 0-ISC00 // 10
 Andi 	T		, 0b11111100
 Ori	T		, 0b00000010 
 Out	MCUCR	, T	
.EndMacro
; INT0_OnH - prerusenie ak INT vojde do vysokej urovni
.Macro INT0_OnH
 In		T		, MCUCR
 ; 1-ISC01, 0-ISC00 // 11
 Ori	T		, 0b00000011 
 Out	MCUCR	, T	
.EndMacro
; INT0_OnChange - prerusenie ak INT zmeni uroven 
.Macro INT0_OnChange
 In		T		, MCUCR
 ; 1-ISC01, 0-ISC00 // 01
 Andi 	T		, 0b11111100
 Ori	T		, 0b00000001 
 Out	MCUCR	, T	
.EndMacro
; INT0_Init - Povoli prerusenie od nabeznej hrany a nastvy port.
.Macro INT0_Init
 .message "INT0_Init: PD2 has been changed to Input. Use Interupt_INT0"
 Cbi	DDRD	, 2
 INT0_OnH
 INT0_Enable
.EndMacro
; - INT1
; INT1_Enable - Povoli prerusenie
.Macro INT1_Enable
 Set
 In		T		, GICR
 Bld	T		, INT1
 Out	GICR	, T
.EndMacro
; INT1_Disable - Zakaze prerusenie
.Macro INT1_Disable
 Clt
 In		T		, GICR
 Bld	T		, INT1
 Out	GICR	, T
.EndMacro
; INT1_StayOnL - prerusenie ak je INT v nizkej urovni
.Macro INT1_StayOnL
 In		T		, MCUCR
 ; 3-ISC11, 2-ISC10  // 00
 Andi 	T		, 0b11110011 
 Out	MCUCR	, T	
.EndMacro
; INT1_OnL - prerusenie ak INT vojde do nizkej urovni
.Macro INT1_OnL
 In		T		, MCUCR
 ; 3-ISC11, 2-ISC10 // 10
 Andi 	T		, 0b11110011
 Ori	T		, 0b00001000 
 Out	MCUCR	, T	
.EndMacro
; INT1_OnH - prerusenie ak INT vojde do vysokej urovni
.Macro INT1_OnH
 In		T		, MCUCR
 ; 3-ISC11, 2-ISC10 // 11
 Ori	T		, 0b00001100 
 Out	MCUCR	, T	
.EndMacro
; INT1_OnChange - prerusenie ak INT zmeni uroven 
.Macro INT1_OnChange
 In		T		, MCUCR
 ; 3-ISC11, 2-ISC10 // 01
 Andi 	T		, 0b11110011
 Ori	T		, 0b00000100 
 Out	MCUCR	, T	
.EndMacro
; INT1_Init - Povoli prerusenie od nabeznej hrany a nastvy port.
.Macro INT1_Init
 .message "INT1_Init: PD3 has been changed to Input. Use Interupt_INT1"
 Cbi	DDRD	, 3
 INT1_OnH
 INT1_Enable
.EndMacro
; - INT2 (iba v puzdre DIP40)
; INT2_Enable - Povoli prerusenie
.Macro INT2_Enable
 Set
 In		T		, GICR
 Bld	T		, INT2
 Out	GICR	, T
.EndMacro
; INT2_Disable - Zakaze prerusenie
.Macro INT2_Disable
 Clt
 In		T		, GICR
 Bld	T		, INT2
 Out	GICR	, T
.EndMacro
; INT2_OnL - prerusenie ak INT vojde do nizkej urovni
.Macro INT2_OnL
 In		T		, MCUCSR
 ; 6-ISC2 // 0
 Andi 	T		, 0b10111111
 Out	MCUCSR	, T	
.EndMacro
; INT2_OnH - prerusenie ak INT vojde do vysokej urovni
.Macro INT2_OnH
 In		T		, MCUCSR
 ; 6-ISC2 // 1
 Ori 	T		, 0b01000000
 Out	MCUCSR	, T	
.EndMacro
; INT2_Init - Povoli prerusenie od nabeznej hrany a nastvy port.
.Macro INT2_Init
 .message "INT2_Init: PB2 has been changed to Input. Use Interupt_INT2"
 Cbi	DDRB	, 2
 INT2_OnH
 INT2_Enable
.EndMacro

; Delay --------------------------------------------------------- Delay
; Delay v us.
.Macro Delay_us
 .message "Delay_us: Error of parametr in uSec. Delay_us [Word] max4095"
.EndMacro
.Macro Delay_us_i
 Push 	H					; 2 cyky
 Push	L					; 2
.if CPUFreqenci = f1Mhz
 Ldi	L		, Low(((@0-13)/4)+1)	; 1 cyklus
 Ldi	H		, High(((@0-13)/4)+1)	; 1 cyklus
.elif CPUFreqenci = f8Mhz
 Ldi	L		, Low((((@0*8)-13)/4)+1)	; 1 cyklus
 Ldi	H		, High((((@0*8)-13)/4)+1)	; 1 cyklus 
.elif CPUFreqenci = f16Mhz
 Ldi	L		, Low((((@0*16)-13)/4)+1)	; 1 cyklus
 Ldi	H		, High((((@0*16)-13)/4)+1)	; 1 cyklus
.endif 
 Delay_us_Loop_%:
  Sbiw  L		,1			; 2 cyklus
  Brne	Delay_us_Loop_%    	; 1 nakoniec, inac 2 cykly
 Pop	L					; 2
 Pop	H 					; 2
.EndMacro
; Delay - telo zo vstupom HL
.Macro Delay_Body
 Delay_us_Body_%:
  Sbiw  L		,1			; 2 cyklus
  Brne	Delay_us_Body_%    	; 1 nakoniec, inac 2 cykly
.EndMacro
.Macro Delay_Parameter
 .message "Delay_Parameter: Error of parameter. [word] max 4095"
.EndMacro
.Macro Delay_Parameter_i
.if CPUFreqenci = f1Mhz
 Ldi	L		, Low(((@0-13)/4)+1)	; 1 cyklus
 Ldi	H		, High(((@0-13)/4)+1)	; 1 cyklus
.elif CPUFreqenci = f8Mhz
 Ldi	L		, Low((((@0*8)-13)/4)+1)	; 1 cyklus
 Ldi	H		, High(((@0*8)-13)/4)+1)	; 1 cyklus 
.elif CPUFreqenci = f16Mhz
 Ldi	L		, Low((((@0*16)-13)/4)+1)	; 1 cyklus
 Ldi	H		, High((((@0*16)-13)/4)+1)	; 1 cyklus
.endif 
.EndMacro
; 1 -> 13 cykly 
; 2 -> 13 + 4 cykly 
; 3 -> 13 + 8 cykly


; --------------------------------------------------------------- Timer
; Makra casovacov
; - Timer 1
; Inicializacia Timera
.Macro Timer_Init
 .message "Timer: Div 8, Max is in register HL (R24:R23)"
 ; Nastavenia zdroju signalu TCCR1B
 ; 0 - Preddelicka :
 ; 1 - 0, /1, /8, /64, /256, /1024, T1, nT1
 ; 2 - 
 ; -----------------------------------------
 ; 3 - 1 rezim casovaca
 ; 4 - 0
 ; -----------------------------------------
 ; 5 -
 ; 6 -
 ; 7 -
 Ldi 	T		, 0b00001010   	; predelicka na /8 ----------------- 3.D   		 
 Out 	TCCR1B	, T	
 ; Nastavenia zdroju signalu TCCR1A
 Ldi 	T		, 0b00000000       		 
 Out 	TCCR1A	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b00010000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T	
 Out	TCNT1H	, T     		; Horny byte
 Out	TCNT1L	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Out 	OCR1AH	, H     		; Horny byte
 Out 	OCR1AL	, L     		; Dolny byte
.EndMacro
 ; Timerove konstanty delicky 
 .Equ Timer_Disable	= 0
 .Equ Timer_Div1	= 1
 .Equ Timer_Div8	= 2
 .Equ Timer_Div64	= 3
 .Equ Timer_Div256	= 4
 .Equ Timer_Div1024	= 5
 .Equ Timer_T1		= 6
 .Equ Timer_NegT1	= 7 
 ; Makro inicializacie timeru s parametrom
; - Timer_Init [Top: Word, Div: Byte (0-7)] 
.Macro Timer_Init_i_i
 ; Nastavenie typu timeru a preddelicky
 Ldi	T		, @1
 Ori 	T		, 0b00001000   	; Maska CTC casovaca   		 
 Out 	TCCR1B	, T	
 ; Nastavenia zdroju signalu TCCR1A
 Ldi 	T		, 0b00000000       		 
 Out 	TCCR1A	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b00010000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T
 Out	TCNT1H	, T     		; Horny byte
 Out	TCNT1L	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Ldi	T		, High(@0)	
 Out 	OCR1AH	, T     		; Horny byte
 Ldi	T		, Low(@0)
 Out 	OCR1AL	, T     		; Dolny byte
.EndMacro 
; Reset Timera 
.Macro Timer_Reset
 PushA
 Interupt_Disable
  Clr	T	
  Out	TCNT1H	, T     		; Horny byte
  Out	TCNT1L	, T     		; Dolny byte
 PopA
.EndMacro 
; Nastatvenie TOPu casovaca z HL
.Macro Timer_Top
  Out 	OCR1AH	, H     		; Horny byte
  Out 	OCR1AL	, L     		; Dolny byte
.EndMacro 
; Timer_Top [Top: Word]
.Macro Timer_Top_i
 Ldi	T		, High (@0)	
 Out 	OCR1AH	, H     		; Horny byte
 Ldi	T		, Low (@0)
 Out 	OCR1AL	, L     		; Dolny byte
.EndMacro 
; Preddelicka
.Macro Timer_Div_i
 ; Nastavenie typu timeru a preddelicky
 Ldi	T		, @0
 Ori 	T		, 0b00001000   	; Maska CTC casovaca   		 
 Out 	TCCR1B	, T	
.EndMacro 
; Zapnutie/Vypnutie timera
.Macro Timer_Enable
 In		T		, TIMSK
 Ori 	T		, 0b00010000       		
 Out 	TIMSK	, T
.EndMacro
.Macro Timer_Disable
 In		T		, TIMSK
 Andi 	T		, 0b11101111       		
 Out 	TIMSK	, T
.EndMacro

; Poznamka:
; 16Mhz : 8		 = 2Mhz Preddelicka
; 2Mhz  : 20 000 = 100Hz
;         20.000 = 4E 20
; Druhy 8bit Timer
; - Timer 2
; Timer8
.Macro Timer8_Init; ( L : Byte ) - L je TOP Citaca. @8Mhz -> 1Mhz / L
 .message "Timer8: Div 8, Top je nastaveny registrom L (R23)"
 ; Nastavenia zdroju signalu TCCR2
 ; 0 - Preddelicka :
 ; 1 - 0, /1, /8, /32, /64, /128, /256, /1024
 ; 2 - 
 ; -----------------------------------------
 ; 3 - 1 rezim casovaca
 ; 4 - 0
 ; 5 - 0
 ; 6 - 0 rezim casovaca
 ; 7 - 0
 Ldi 	T		, 0b00001010   	; predelicka na /8 ----------------- 3.E   		 
 Out 	TCCR2	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b10000000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T	
 Out	TCNT2	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Out 	OCR2	, L     		; byte
.EndMacro 
 ; Timerove konstanty delicky 
 .Equ Timer8_Disable	= 0
 .Equ Timer8_Div1		= 1
 .Equ Timer8_Div8		= 2
 .Equ Timer8_Div32		= 3
 .Equ Timer8_Div64		= 4
 .Equ Timer8_Div128		= 5
 .Equ Timer8_Div256		= 6
 .Equ Timer8_Div1024	= 7 
 ; Makro inicializacie timeru s parametrom
; - Timer8_Init [Top: Byte, Div: Byte (0-7)] 
.Macro Timer8_Init_i_i 
 Ldi	T		, @1
 Ori 	T		, 0b00001000   	; Maska CTC casovaca   		 
 Out 	TCCR2	, T	
 ; Nastavenie prerusenia TIMSK
 ; prerusenie od OCIE1A
 In		T		, TIMSK
 Ori 	T		, 0b10000000       		
 Out 	TIMSK	, T
 ; Reset Pocitadla
 Clr	T	
 Out	TCNT2	, T     		; Dolny byte
 ; Nastavenie hodnoty pre register OCR1A
 Ldi	T		, @0
 Out 	OCR2	, T    		; byte
.EndMacro 
; Reset Timera 
.Macro Timer8_Reset
 PushA
 Interupt_Disable
 Clr	T	
 Out	TCNT2	, T
 PopA     	
.EndMacro  
; Nastatvenie TOPu casovaca z L
.Macro Timer8_Top
  Out 	OCR2	, L     		; Dolny byte
.EndMacro 
; Timer_Top [Top: Byte]
.Macro Timer8_Top_i
 Ldi	T		, @0
 Out 	OCR2	, L     		; Dolny byte
.EndMacro 
; Preddelicka
.Macro Timer8_Div_i
 ; Nastavenie typu timeru a preddelicky
 Ldi	T		, @0
 Ori 	T		, 0b00001000   	; Maska CTC casovaca   		 
 Out 	TCCR2	, T	
.EndMacro 

; -------------------------------------------------------------- EEPROM
; Cakanie, kym je pamat pripravena
.Macro EEPROM_Wait
eeWait_%:
 Sbic	EECR	, EEWE			; Ak je eeprom zanepraznena
 Rjmp	eeWait_%				; Cakaj!
.EndMacro
; Citanie
.Macro EEPROM_Read ; ( HL : Word - Adreasa; A : Byte - Data )
 EEPROM_Wait
 Out	EEARH	, H				; Vyslanie do EEPROM
 Out	EEARL	, L				; Vyslanie do EEPROM
 Sbi	EECR	, EERE			; Vlajka citania
 In		A		, EEDR			; Nacitanie spodneho bytu
.EndMacro
; Citanie z Adresy
; EEPROM_Read [word]
.Macro EEPROM_Read_i ; ( Word - Adreasa; A : Byte - Data )
eeWaitr_%:
 Sbic	EECR	, EEWE			; Ak je eeprom zanepraznena
 Rjmp	eeWaitr_%				; Cakaj!
 Push	B
 Ldi	T		, Low(@0)
 Ldi	B		, High(@0) 
 Out	EEARH	, B				; Vyslanie do EEPROM
 Out	EEARL	, T				; Vyslanie do EEPROM
 Sbi	EECR	, EERE			; Vlajka citania
 Pop	B
 In		A		, EEDR			; Nacitanie spodneho bytu
.EndMacro
; Zapis
.Macro EEPROM_Write ; ( HL : Word - Adreasa) : A  - Data 
 EEPROM_Wait
 In		T		, SREG
 Cli
 Out	EEARH	, H				; Vyslanie do EEPROM
 Out	EEARL	, L				; Vyslanie do EEPROM
 Out	EEDR	, A				; Vyslanie do EEPROM
 Sbi	EECR	, EEMWE			; Vlajka zapisu
 Sbi	EECR	, EEWE			; Vlajka zapisu
 Out	SREG	, T
.EndMacro
; Zapis
; - EEPROM_Write [Word]
.Macro EEPROM_Write_i ; ( Word - Adreasa) : A  - Data 
eeWaitw_%:
 Sbic	EECR	, EEWE			; Ak je eeprom zanepraznena
 Rjmp	eeWaitw_%				; Cakaj!
 In		T		, SREG
 Cli
 Push	H						; Ulozit HL
 Push	L
 Ldi	H		, High(@0)		; Nacitanie adresy do HL
 Ldi	L		, Low(@0)
 Out	EEARH	, H				; Vyslanie do EEPROM
 Out	EEARL	, L				; Vyslanie do EEPROM
 Out	EEDR	, A				; Vyslanie do EEPROM
 Sbi	EECR	, EEMWE			; Vlajka zapisu
 Sbi	EECR	, EEWE			; Vlajka zapisu
 Pop	L
 Pop	H
 Out	SREG	, T
.EndMacro

;--------------------------------------------------------- AD prevodnik
; AD Prevodnik
; - Volanie
.Macro	AD_Convert
  In	T		, ADMUX
  Andi	T		, 0b11100000		; Maska Mux
  Ori	T		, 0b00100000		; 8Bit
  Or	T		, A					; Maska MUX
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Pockaj
  Delay_us [10]  
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	A		, ADCL
   IN	A		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volanie s parametrom
.Macro	AD_Convert_i
 Ldi	A		, @0
  In	T		, ADMUX
  Andi	T		, 0b11100000		; Maska Mux
  Ori	T		, 0b00100000		; 8Bit
  Or	T		, A					; Maska MUX  
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:  
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	A		, ADCL
   IN	A		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volanie s parametrom
.Macro	AD_Convert_i_8
 .Message "AD_Convert: Return register must by R16-R31"
 Ldi	A		, @0
  In	T		, ADMUX
  Andi	T		, 0b11100000		; Maska Mux
  Ori	T		, 0b00100000		; 8Bit
  Or	T		, A					; Maska MUX
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	@1		, ADCL
   IN	@1		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volanie s parametrom
.Macro	AD_Convert_8_8
 .Message "AD_Convert: Return register must by R16-R31"
  In	T		, ADMUX
  Andi	T		, 0b11100000		; Maska Mux
  Ori	T		, 0b00100000		; 8Bit
  Or	T		, @0				; Maska MUX
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	@1		, ADCL
   IN	@1		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volanie
.Macro	AD_Convert_10
  In	T		, ADMUX
  Andi	T		, 0b11000000		; Maska Mux
  Or	T		, A					; Maska MUX
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	L		, ADCL
   IN	H		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volanie s parametrom
.Macro	AD_Convert_10_i
 Ldi	A		, @0
  In	T		, ADMUX
  Andi	T		, 0b11000000		; Maska Mux
  Or	T		, A					; Maska MUX
  Out	ADMUX	, T
  ; Reset
  Ldi	T		, 0b10000000		; ADEN
  Out	ADCSRA	, T
  ; Run
  Ldi	T		, 0b11000000		; ADStart
  Out	ADCSRA	, T
  ; Wait
  AD_Convert_Wait_%:
   Sbis	ADCSRA	, ADIF
   Rjmp	AD_Convert_Wait_%
  ; Nacitanie 8 bit
   IN	L		, ADCL
   IN	H		, ADCH
  ; Vynulovanie ADIF
  Sbi	ADCSRA	, ADIF	 
  ; Vypnute AD prevodnika
  Ldi	T		, 0b00000000		; ADEN - Off
  Out	ADCSRA	, T
.EndMacro
; - Volba referencie
.MACRO AD_Ref5V	
 Ldi	T		, 0b01000000 
 Out	ADMUX	, T
.ENDMACRO 
.MACRO AD_Ref2V56	
 Ldi	T		, 0b11000000 
 Out	ADMUX	, T
.ENDMACRO 
.MACRO AD_ARef	
 Ldi	T		, 0b00000000 
 Out	ADMUX	, T
.ENDMACRO 
; USART --------------------------------------------------------- USART
.Macro USART_Init;
 .message "USART_Init: Error of parameter! USART_Init [UBRR]"
.EndMacro
; Tabulka rychlosti:
.if CPUFreqenci = f1Mhz
 .Equ U9600 = 6
 .Equ U2400 = 25
 .Equ U4800	= 12
.elif CPUFreqenci = f8Mhz
 .Equ U9600 = 51
 .Equ U2400 = 207
 .Equ U4800	= 103
 .Equ U19K2 = 25
 .Equ U38K4 = 12
 .Equ U250K = 1
 .Equ U500K	= 0 
.elif CPUFreqenci = f16Mhz
 .Equ U9600 = 103
 .Equ U2400 = 416
 .Equ U4800	= 207
 .Equ U19K2 = 51
 .Equ U38K4 = 25
 .Equ U250K = 3
 .Equ U500K	= 1
 .Equ U1M	= 0 
.endif
; USART INIT
.Macro USART_Init_i;
 Ldi	T		, 0x00
 Out	UCSRC	, T
 Ldi	T		, @0		; Nadtavenie rychlosti ----------------- 8.A
 Out	UBRRL	, T			; - Pre @16Mhz je hodnota 3
 ; === UCSRB ===
 ; 7. RXCIE		- Povolenie prerusenia od dokoncenia prijimania
 ; 6. TXCIE		- Povolenie prerusenia od dokoncenia Vysielania
 ; 5. UDRIE
 ; ------------------------------------------------------------
 ; 4. RXEN 		- Povolenie Prijimania
 ; 3. TXEN     	- Povolenie Vysielania
 ; ------------------------------------------------------------
 ; 2. UCSZ2		- Povolenie 9 bitu
 ; 1. RXB8		- 9bit prijimaca
 ; 0. TXB8		- 9bit vysielaca
 Ldi	T		, 0b00000000; Povolenie Vysielania/prijimania
 Out	UCSRB	, T
 ; === UCSRC ===
 ; 7. URSEL = 1
 ; 6. UMSEL		- 1-Synchronizovany/0-Asynchronizovany
 ; -------------------------------------------------------------
 ; 5. UPM1		- Parita:
 ; 4. UPM0		- Disable/Reserved/Even(parny)/Odd(Neparny)
 ; -------------------------------------------------------------
 ; 3. USBS		- Stop bit. 0-1x / 1-2x
 ; -------------------------------------------------------------
 ; 2. UCSZ1		- Pocet bitov:
 ; 1. UCSZ0		- 5,6,7,8 Bit(UCSZ2=1: Reserved/Reserved/Reserved/9Bit)
 ; -------------------------------------------------------------
 ; 0. UCPOL		- Polarita : 0-nabezna / 1-klesajuca
 Ldi	T		, 0b10000110; 
 Out	UCSRC	, T 
.EndMacro 
; Odosielanie a prijem
.Macro	USART_Send ; ( A - Data )
 wUSART_WaitLoopS:
 Sbis	UCSRA	, UDRE
 Rjmp	wUSART_WaitLoopS
 Out	UDR		, A
.EndMacro
.Macro	USART_Send_i ; ( Byte - Data )
 wUSART_WaitLoopS:
 Sbis	UCSRA	, UDRE
 Rjmp	wUSART_WaitLoopS
 Ldi	T		, @0
 Out	UDR		, T
.EndMacro
.Macro	USART_Send_8 ; ( Register - Data )
 wUSART_WaitLoopS:
 Sbis	UCSRA	, UDRE
 Rjmp	wUSART_WaitLoopS
 Out	UDR		, @0
.EndMacro
.Macro	USART_Read ; : A - Data 
 wUSART_WaitLoopR:
 Sbis	UCSRA	, RXC
 Rjmp	wUSART_WaitLoopR
 In		A		, UDR
.EndMacro
.Macro	USART_Read_8 ; : Register - Data 
 wUSART_WaitLoopR:
 Sbis	UCSRA	, RXC
 Rjmp	wUSART_WaitLoopR
 In		@0		, UDR
.EndMacro
 ; === UCSRA === STATOVY Register ====
 ; 7. RXC		- Prijatie ukoncene
 ; 6. TXC		- Vysielanie dokoncene
 ; 5. UDRE		- Vysielaci register je prazny
 ; -------------------------------------------------------------
 ; 4. FE		- Frame Error
 ; 3. DOR		- Data OverRun
 ; 2. PE		- Parity Error
 ; -------------------------------------------------------------
 ; 1. U2X		- Dvojnasobna rychlost
 ; 0. MPCM		- Multyporocesor mode
 ; Utility
.Macro USART_Rx_Start
 Sbi	UCSRB	, RXEN
.EndMacro
.Macro USART_Rx_Stop
 Cbi	UCSRB	, RXEN
.EndMacro
.Macro USART_Tx_Start
 Sbi	UCSRB	, TXEN
.EndMacro
.Macro USART_Tx_Stop
 Cbi	UCSRB	, TXEN
.EndMacro
.Macro USART_Start
 Sbi	UCSRB	, RXEN
 Sbi	UCSRB	, TXEN
.EndMacro
.Macro USART_Stop
 Cbi	UCSRB	, RXEN
 Cbi	UCSRB	, TXEN
.EndMacro
.Macro USART_SkipIfReceive
 Sbis	UCSRA	, RXC
.EndMacro
.Macro USART_SkipIfNotReceive
 Sbic	UCSRA	, RXC
.EndMacro
.Macro USART_SkipIf_FrameError
 Sbis	UCSRA	, FE
.EndMacro
.Macro USART_SkipIfNot_FrameError
 Sbic	UCSRA	, FE
.EndMacro
.Macro USART_SkipIf_DataOverRun
 Sbis	UCSRA	, DOR
.EndMacro
.Macro USART_SkipIfNot_DataOverRun
 Sbic	UCSRA	, DOR
.EndMacro
.Macro USART_ResetFlag_FrameError
 Cbi	UCSRA	, FE
.EndMacro
.Macro USART_ResetFlag_DataOverRun
 Cbi	UCSRA	, DOR
.EndMacro
; Nastavenia
.Macro USART_Interupt_RX_Enable
 Sbi	UCSRB	, RXCIE
.EndMacro
.Macro USART_Interupt_RX_Disable
 Cbi	UCSRB	, RXCIE
.EndMacro
.Macro USART_Interupt_TX_Enable
 Sbi	UCSRB	, TXCIE
.EndMacro
.Macro USART_Interupt_TX_Disable
 Cbi	UCSRB	, TXCIE
.EndMacro
 
; Utility -------------------------------------------------------------

; Vloz na adresu Z
.Macro Poke 
 St Z+	, A
.EndMacro 
.Macro Poke_8 
 St Z+	, @0
.EndMacro 
.Macro Poke_i
 Ldi T	, @0 
 St Z+	, T
.EndMacro 
.Macro Peek 
 Ld A	, Z+
.EndMacro 
.Macro Peek_8 
 Ld @0	, Z+
.EndMacro 

; I2C
; TWCR : 7:TWINT, 6:TWEA, 5:TWSTA, 4:TWSTO, 3:TWWC, 2: TWEN, 1:, 0:TWIE

.Macro I2C_Init
 Cbi PortC	, 5
 Cbi PortC	, 4
 Ldi T	 	, 0	
 Out TWSR	, T
 Ldi T		, 72 ; @16Mhz 100.000-72 (400 - 12)
 Out TWBR   , T 
.EndMacro

; Procedura: Start : T (priznak chyby)
aI2C_Start:
  Clt            				; Vynulovanie T
  Ldi   T 		, 0b10100100	;(1<<TWINT)+(1<<TWSTA)+(1<<TWEN)
  Out   TWCR	, T
 ; Cakaj 
 I2C_StartWait:
   In   T		, TWCR        	;Cakanie na odoslanie STRARTu   
   Sbrs T		, TWINT
    RJmp   I2C_StartWait
; Status   
   In   T		, TWSR         	; Nacitanie Statusu
   Andi T		, 0xF8			; Maska preddelicky
   Cpi  T 		, 0x08			; Ak je status: I2C_STATUS_START
    Breq   I2C_StartEnd     	; OK       
   Cpi  T 		, 0x10			; Ak je status: I2C_STATUS_RESTART  
    Breq   I2C_StartEnd	      	; OK
   Set 				           	; Inak zdvihni vlajku chyby   
 I2C_StartEnd:
 Ret
.Macro I2C_Start
 RCall aI2C_Start
.EndMacro

; Procedura: Stop : T (priznak chyby)
aI2C_Stop:
 Clt
aI2C_Stop_NoChangeT:
 Ldi  T 		, 0b10010100	;(1<<TWINT)+(1<<TWSTO)+(1<<TWEN)
 Out  TWCR		, T
 Ret     
.Macro I2C_Stop
 RCAll aI2C_Stop
.EndMacro
.Macro I2C_Stop_NoChangeT
 RCAll aI2C_Stop_NoChangeT
.EndMacro
 
; Odoslanie adresy
aI2C_SendAdr: 
  Bst   A		, 0        		; Ulozenie R/w bit do T
  ;Andi  A		, 0b11111110
  Out   TWDR	, A				; Ulozenie A do datoveho registra
  Ldi   T 		, 0b10000100	;(1<<TWINT)+(1<<TWEN)
  Out   TWCR	, T
 ; Cakanie na odoslanie adresy 
 I2C_SendAdrWait:
  In   T		, TWCR         	; Cakanie, kym sa odosle adresa
  Sbrs T		, TWINT
   Rjmp I2C_SendAdrWait
 ; Nacitanie statusu  
  In   T		, TWSR  	    ; Nacitanie statusu
  Andi T		, 0xF8   	    ; Maska preddelicky
  Brts I2C_SendAdrRead      	; Citanie adresy
  ; Zapis Adresy
  Clt				            ; clear Err flag
  Cpi  T 		, 0x18			; I2C_STATUS_MT_SLA_ACK_W
								; Ak je prijaty ACK z slave
   Breq I2C_SendAdrEnd	       	; OK 
  ; Chyba a pozastavenie spojenia      
  Set 				            ; Inac Chyba
  Cpi  T		, 0x20			; I2C_STATUS_MT_SLA_NACK_W
								; Ak neprisiel ACK z slave
   Breq aI2C_Stop_NoChangeT		; Ukoncenie komunikacie 
 ; Koniec odoslania  
 I2C_SendAdrEnd:
  Ret
 ; Nacitanie adresy 
 I2C_SendAdrRead:
  Clt 				           	; Zmazanie vlajku chyby
  Cpi  T		, 0x40			; I2C_STATUS_MT_SLA_ACK_R
								; Ak je prijaty ACK z slave
   Breq I2C_SendAdrEnd     		; OK       
  ; Chyba
  Set            ;ERROR
  Cpi  T		, 0x48			; I2C_STATUS_MT_SLA_NACK_R
								; Ak neprisiel ACK z slave
   Breq aI2C_Stop_NoChangeT		; Ukoncenie komunikacie       
  RJmp   I2C_SendAdrEnd
.Macro I2C_Adress
 RCall aI2C_SendAdr
.EndMacro 
.Macro I2C_Adress_i
 Ldi A		, @0
 RCall aI2C_SendAdr
.EndMacro 
.Macro I2C_SendAdr
 RCall aI2C_SendAdr
.EndMacro 
.Macro I2C_SendAdr_i
 Ldi A		, @0
 RCall aI2C_SendAdr
.EndMacro 

; Odoslanie dat
aI2C_SendData:
  Clt
  Out   TWDR	, A				; Ulozenie A do datoveho registra
  Ldi   T 		, 0b10000100	;(1<<TWINT)+(1<<TWEN)
  Out   TWCR	, T
 ; Slucka cakania
 I2C_SendDataWait:
  In    T		, TWCR          ; Cakanie kym odosle
  Sbrs  T		, TWINT
   RJmp  I2C_SendDataWait
  ; Nacitanie statusu 
  In    T		, TWSR         	; Nacitanie statusu
  Andi  T		, 0xF8         	; Maska preddelicky
  Cpi   T		, 0x28			; I2C_STATUS_MT_DATA_ACK_W
								; Ak je prijaty ACK z slave
   Breq  I2C_SendDataEnd      	; OK       
  ; Inak chyba
  Set				            ; Vlajka chyby
  Cpi   T		, 0x30			; I2C_STATUS_MT_DATA_NACK_W
								; Ak neprisiel ACK z slave
   Breq aI2C_Stop_NoChangeT		; Ukoncenie komunikacie       
 I2C_SendDataEnd:
 Ret
.Macro I2C_SendData
 RCall aI2C_SendData
.EndMacro 
.Macro I2C_SendData_i
 Ldi  A		, @0
 RCall aI2C_SendData
.EndMacro 
.Macro I2C_Send
 RCall aI2C_SendData
.EndMacro 
.Macro I2C_Send
 Ldi  A		, @0
 RCall aI2C_SendData
.EndMacro 
.Macro I2C_Send_8
 Mov  A		, @0
 RCall aI2C_SendData
.EndMacro 

; Nacitanie dat
aI2C_ReadData:
  Ldi   A		, 0x00
aI2C_ReadData_Or:
  Clt
  Ori   A 		, 0b10000100	;(1<<TWINT)+(1<<TWEN)
  Out   TWCR	, A
 ; Slucka cakania
 I2C_ReadDataWait:
  In    T 		, TWCR          ; Cakanie kym nepridu data
  Sbrs  T		, TWINT
   RJmp I2C_ReadDataWait
  ; Status
  In    T		, TWSR 	        ; Nacitanie statusu
  Andi  T		, 0xF8	        ; Maska preddelicky
  Cpi   T		, 0x50			; I2C_STATUS_MT_DATA_ACK_R
								; Ak je prijaty ACK z slave
   Breq I2C_ReadDataEnd      	; OK       
  ; Nepotvrdeny prijem
  Cpi   T		, 0x58			; I2C_STATUS_MT_DATA_NACK_R
								; Ak neprisiel ACK z slave
   Breq I2C_ReadDataEnd      	; OK       
  ; Inac vyhlas chybu
  Set				            ; Vlajka chyby
 I2C_ReadDataEnd:
  In   A		, TWDR          ; NAcitanie bytu
  Ret
.Macro I2C_ReadData
 RCall aI2C_ReadData
.EndMacro  
.Macro I2C_Read
 RCall aI2C_ReadData
.EndMacro  
.Macro I2C_Read_8
 RCall aI2C_ReadData
 Mov @0		, A
.EndMacro  
.Macro I2C_Next
 Ldi   A  ,0b01000000				;(1<<TWEA)
 RCall aI2C_ReadData_Or
.EndMacro  

; --------------------------------------------------------------------
; Inicializacia
 Begin:
  
 ; Zasobnik
 Ldi	T	, High(RAMEND)	; Adresa konca pamati H
 Out	SPH	, T
 Ldi	T 	, Low(RAMEND)	; Adresa konca pamati L
 Out	SPL	, T
 

 
